package org.eco.als.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.bo.SmTaskSubBo;
import org.eco.als.domain.vo.SmTaskSubImportVo;
import org.eco.als.domain.vo.SmTaskSubVo;
import org.eco.als.service.ISmTaskSubService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.utils.SpringUtils;
import org.eco.common.core.utils.ValidatorUtils;
import org.eco.common.excel.core.ExcelListener;
import org.eco.common.excel.core.ExcelResult;
import org.eco.common.excel.entity.ExcelResultRes;

import java.util.List;

/**
 * 定检任务子自定义导入
 *
 * @author wgk
 */
@Slf4j
public class SmTaskSubImportListener extends AnalysisEventListener<SmTaskSubImportVo> implements ExcelListener<SmTaskSubImportVo> {
    private final ISmTaskSubService smTaskSubService;

    private final Boolean isUpdateSupport;
    private final LoginUser loginUser;
    private final StringBuilder successMsg = new StringBuilder();
    private final StringBuilder failureMsg = new StringBuilder();
    private int successNum = 0;
    private int failureNum = 0;

    public SmTaskSubImportListener(Boolean isUpdateSupport, LoginUser loginUser) {
        this.smTaskSubService = SpringUtils.getBean(ISmTaskSubService.class);
        this.isUpdateSupport = isUpdateSupport;
        this.loginUser = loginUser;
    }

    @Override
    public void invoke(SmTaskSubImportVo smTaskSubVo, AnalysisContext context) {
        try {

            SmTaskSubBo smTaskSubBo = BeanUtil.toBean(smTaskSubVo, SmTaskSubBo.class);

            //TODO:根据某个字段，查询数据库表中是否存在记录，不存在就新增，存在就更新
            SmTaskSubVo smTaskSubVo1 = null;

            //smTaskSubVo1 = smTaskSubService.selectBySomefield(smTaskSubVo.getSomefield());
            if (ObjectUtil.isNull(smTaskSubVo1)) {
                //不存在就新增
                setBo(smTaskSubBo);
                ValidatorUtils.validate(smTaskSubBo);
                boolean inserted = smTaskSubService.insert(smTaskSubBo);

                if (inserted) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、定检任务子 记录导入成功");
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、定检任务子 记录导入失败");
                }
            } else if (isUpdateSupport) {
                //存在就更新
                smTaskSubBo.setId(smTaskSubVo1.getId());//主键
                smTaskSubBo.setVersion(smTaskSubVo1.getVersion());
                boolean updated = smTaskSubService.update(smTaskSubBo);
                if (updated) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、定检任务子 记录更新成功");
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、定检任务子 记录更新失败");
                }
            }
        } catch (Exception e) {
            failureNum++;
            String msg = "<br/>" + failureNum + "、定检任务子 记录导入失败：";
            failureMsg.append(msg).append(e.getMessage());
            log.error(msg, e);
        }
    }

    private void setBo(SmTaskSubBo smTaskSubBo) {
        smTaskSubBo.setVersion(0);
        smTaskSubBo.setCreateBy(loginUser.getUserId());
        smTaskSubBo.setUpdateBy(loginUser.getUserId());
        smTaskSubBo.setTenantId(loginUser.getTenantId());
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        log.info("解析完成");
    }

    @Override
    public ExcelResult<SmTaskSubImportVo> getExcelResult() {
        return new ExcelResult<>() {

            @Override
            public ExcelResultRes getAnalysis() {
                if (failureNum > 0 && successNum == 0) {
                    failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据没有成功导入，错误如下：");
                    return ExcelResultRes.builder().logInfo(failureMsg.toString()).status("0").build();
                } else if (failureNum > 0 && successNum > 0) {
                    failureMsg.insert(0, "很抱歉，部分导入失败！共 " + failureNum + " 条数据没有成功导入，错误如下：");
                    return ExcelResultRes.builder().logInfo(failureMsg.toString()).status("2").build();
                } else {
                    successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
                    return ExcelResultRes.builder().logInfo(successMsg.toString()).status("1").build();
                }
            }

            @Override
            public List<SmTaskSubImportVo> getList() {
                return null;
            }

            @Override
            public List<String> getErrorList() {
                return null;
            }
        };
    }
}
