package org.eco.als.service;

import org.eco.als.domain.SmTaskSub;
import org.eco.als.domain.bo.SmTaskSubBo;
import org.eco.als.domain.vo.SmTaskSubVo;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.IBaseService;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 定检任务子Service接口
 *
 * @author wgk
 * @date 2024-07-25
 */
public interface ISmTaskSubService extends IBaseService<SmTaskSub> {
    /**
     * 查询定检任务子
     *
     * @param id 定检任务子主键
     * @return 定检任务子
     */
    SmTaskSubVo selectById(Long id);

    /**
     * 查询定检任务子列表
     *
     * @param smTaskSubBo 定检任务子Bo
     * @return 定检任务子集合
     */
    List<SmTaskSubVo> selectList(SmTaskSubBo smTaskSubBo);

    /**
     * 分页查询定检任务子列表
     *
     * @param smTaskSubBo 定检任务子Bo
     * @return 分页定检任务子集合
     */
    PageResult<SmTaskSubVo> selectPage(SmTaskSubBo smTaskSubBo);

    /**
     * 新增定检任务子
     *
     * @param smTaskSubBo 定检任务子Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    boolean insert(SmTaskSubBo smTaskSubBo);

    /**
     * 新增定检任务子，前台提供主键值，一般用于导入的场合
     *
     * @param smTaskSubBo 定检任务子Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    boolean insertWithPk(SmTaskSubBo smTaskSubBo);

    /**
     * 修改定检任务子
     *
     * @param smTaskSubBo 定检任务子Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    boolean update(SmTaskSubBo smTaskSubBo);

    /**
     * 批量删除定检任务子
     *
     * @param ids 需要删除的定检任务子主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    boolean deleteByIds(Long[] ids);

    /**
     * 异步导入
     *
     * @param file          导入的文件
     * @param updateSupport 是否覆盖
     * @param user          用户上下文信息
     */
    @Async
    void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser user);

    /**
     * asyncExport 异步导出
     *
     * @param listVo    数据列表
     * @param sheetName 文件名称
     * @param user      上下文
     */
    @Async
    void asyncExport(List<SmTaskSubVo> listVo, String sheetName, LoginUser user);

}
